@@ -54,11 +54,8 @@ c1=hex(min(int(bg_color[1:5],16)+10000, 65535))[2:6]
 c2=hex(min(int(bg_color[5:9],16)+10000, 65535))[2:6]
 c3=hex(min(int(bg_color[9:],16)+10000, 65535))[2:6]
 bg_color = "#" + c1 + c2 + c3
-file = open("/home/user/feedingit_widget.log", "a")
-file.write(str(bg_color)+"\n"+color_style.lookup_color('DefaultBackgroundColor').to_string())
-file.close()
 
-del color_style, c1, c2, d3, settings
+del color_style, c1, c2, c3, settings
 
 CONFIGDIR="/home/user/.feedingit/"
 SOURCE=CONFIGDIR + "source"
