@@ -44,6 +44,8 @@ logger = logging.getLogger(__name__)
 import debugging
 debugging.init(dot_directory=".feedingit", program_name="update_feeds")
 
+from updatedbus import update_server_object
+
 CONFIGDIR="/home/user/.feedingit/"
 #DESKTOP_FILE = "/usr/share/applications/hildon-status-menu/feedingit_status.desktop"
 
@@ -169,6 +171,14 @@ class FeedUpdate(UpdateServerObject):
 
         if not self.am_updating:
             logger.info("Nothing to do for a while.  Quitting.")
+
+            # Make any progress bar go away.
+            try:
+                update_server_object().UpdateProgress(
+                    100, 0, 0, 0, 0, 0, 0, "")
+            except Exception:
+                logger.exception("Sending final progress update")
+
             mainloop.quit()
 
     def StopUpdate(self):
